# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import api, models


class StockMoveLine(models.Model):
    _inherit = "stock.move.line"

    @api.depends("product_id")
    def _compute_expiration_date(self):
        """
        We need to overide this compute to get expiration_date compute in
        lot_id instead of basic calcul made by odoo
        """
        for move_line in self:
            if move_line.lot_id:
                move_line.expiration_date = move_line.lot_id.expiration_date
            else:
                super(StockMoveLine, self)._compute_expiration_date()

    @api.onchange("lot_id")
    def _onchange_lot_id(self):
        """
        We need to override this onchange so that it works both in stock and mrp
        """
        if not self.product_id.use_expiration_date:
            return
        if self.lot_id:
            self.expiration_date = self.lot_id.expiration_date
        else:
            self.expiration_date = False
