# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# Copyright 2017 Open For Small Business Ltd
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
import datetime
import re

from odoo import _, api, fields, models
from odoo.exceptions import UserError


class FumocStockProductionLot(models.Model):
    _inherit = "stock.production.lot"

    # ------------------------------------------------------
    # Business method
    # ------------------------------------------------------
    def _get_dates(self, product_id=None, lotname=None):
        """Replaces existing method."""
        mapped_fields = {
            "expiration_date": "expiration_time",
            "use_date": "use_time",
            "removal_date": "removal_time",
            "alert_date": "alert_time",
        }
        res = dict.fromkeys(mapped_fields, False)
        product = self.env["product.product"].browse(product_id) or self.product_id
        lot_name = lotname or self.name
        if product and lot_name:
            if not self.env.user.company_id.year_reference_lot:
                raise UserError(
                    _("L'année de référence pour les lots n'a pas été " "configurée !")
                )
            suffix = re.findall(r"([\d]+)\D*", lot_name)[-1]
            if len(suffix) >= 4:
                readable_suffix = suffix[-4:]
                year_ref = self.env.user.company_id.year_reference_lot
                year_index = int(readable_suffix[:1])
                year = year_ref + year_index
                quantieme = int(readable_suffix[1:])
                production_date = datetime.datetime(year, 1, 1) + datetime.timedelta(
                    days=quantieme - 1
                )
                for field in mapped_fields:
                    duration = getattr(product, mapped_fields[field])
                    if duration:
                        date = production_date + datetime.timedelta(days=duration)
                        res[field] = fields.Datetime.to_string(date)
        return res

    # Assign dates according to products data
    @api.model_create_multi
    def create(self, vals_list):
        for vals in vals_list:
            dates = self._get_dates(
                vals.get("product_id") or self.env.context.get("default_product_id"),
                vals.get("name") or self.env.context.get("default_name"),
            )
            for d in dates:
                if not vals.get(d):
                    vals[d] = dates[d]
        return super().create(vals_list)

    @api.onchange("name")
    def _onchange_name(self):
        dates_dict = self._get_dates()
        for field, value in dates_dict.items():
            setattr(self, field, value)

    # function copied from https://github.com/odoonz/odoonz-addons/tree/14.0/stock_prodlot_qty
    # Copyright 2017 Open For Small Business Ltd
    def name_get(self):
        res = super().name_get()
        if self.env.context.get("show_qty"):
            res = self.browse([r[0] for r in res]).mapped(
                lambda r: (
                    r.id,
                    r.name
                    and (r.name + " (en stock : %.0f)" % r.product_qty)
                    or r.name,
                )
            )
        return res
